#include "bbxml.h"
#include "bberrors.h"

EXPORT_C TSingleXml::TSingleXml(MNestedXmlHandler* aParent, CXmlParser* aParser, HBufC*& aBuf, MBBData& aValue, TBool aCheckType) : TBBXml(aParent, aParser, aBuf, aValue, aCheckType)
{
}

EXPORT_C TCheckingXml::TCheckingXml(MNestedXmlHandler* aParent, CXmlParser* aParser) :
		iParent(aParent), iIgnoreRest(EFalse), iParser(aParser)
{
}

EXPORT_C TBBXml::TBBXml(MNestedXmlHandler* aParent, CXmlParser* aParser, 
		HBufC*& aBuf, MBBData& aValue, TBool aCheckType) : 
		TCheckingXml(aParent, aParser), iBuf(aBuf), 
		iValue(aValue), iCheckType(aCheckType)
{
	if (iBuf) {
		iBuf->Des().Zero();
	} 
}
EXPORT_C CContainerXml::CContainerXml(MNestedXmlHandler* aParent, CXmlParser* aParser,
		HBufC*& aBuf, MBBData& aValue, TBool aCheckType) :
		TBBXml(aParent, aParser, aBuf, aValue, aCheckType)
{
}

EXPORT_C void TCheckingXml::StartElement(const XML_Char * name,
 	const XML_Char ** atts)
{
	TRAPD(err, StartElementL(name, atts));
	if (err!=KErrNone) {
		ErrorInHandler(err);
	}
}

EXPORT_C void TSingleXml::StartElementL(const XML_Char * name,
 	const XML_Char ** atts)
{
	TPtrC n((const TUint16*)name);
	if (n.Compare(iValue.Name()) ) {
		User::Leave(KNameDoesNotMatch);
	}
	if (iCheckType) {
		TTypeName read_type=TTypeName::IdFromAttributes(atts);
		read_type.CompareMajorL(iValue.Type());
	}
}

EXPORT_C void TCheckingXml::EndElement(const XML_Char * name)
{
	if (!iIgnoreRest) {
		TRAPD(err, EndElementL(name));
		if (err!=KErrNone) {
			ErrorInHandler(err);
		}
	}
	iParent->EndElement(name);
	iParser->SetHandler(iParent);
}


EXPORT_C void TSingleXml::EndElementL(const XML_Char * /*name*/)
{
	if (iBuf) {
		iBuf->Des().Trim();
		iValue.FromStringL(*iBuf);
	} else {
		iValue.FromStringL(KNullDesC);
	}
}
EXPORT_C void TCheckingXml::CharacterData(const XML_Char *s,
 	    int len)
{
	if (!iIgnoreRest) {
		TRAPD(err, CharacterDataL(s, len));
		if (err!=KErrNone) {
			ErrorInHandler(err);
		}
	}
}

EXPORT_C void TCheckingXml::ErrorInHandler(TInt aError)
{
	iIgnoreRest=ETrue;
	iParent->SetError(aError);
}

EXPORT_C void TSingleXml::CharacterDataL(const XML_Char *s,
 	    int len)
{
	if (!iBuf) {
		iBuf=HBufC::NewL(len*2);
	}
	while (iBuf->Length() + len > iBuf->Des().MaxLength()) {
		iBuf=iBuf->ReAllocL(iBuf->Des().MaxLength() * 2);
	}
	TPtrC p((const TUint16*)s, len);
	iBuf->Des().Append(p);
}

EXPORT_C void TSingleXml::Error(XML_Error Code, const XML_LChar * String, long ByteIndex)
{
	iParent->Error(Code, String, ByteIndex);
}

EXPORT_C void TSingleXml::SetError(TInt /*aError*/)
{
	User::Leave(KErrNestedXmlExpectedSingle);
}

EXPORT_C void CContainerXml::StartElementL(const XML_Char *name,
			const XML_Char **atts)
{
	if (iDepth==0) {
		TPtrC n((const TUint16*)name);
		if (n.Compare(iValue.Name()) ) {
			User::Leave(KNameDoesNotMatch);
		}
		if (iCheckType) {
			TTypeName read_type=TTypeName::IdFromAttributes(atts);
			read_type.CompareMajorL(iValue.Type());
		}
	} else {
		StartInnerElementL(name, atts);
	}
	++iDepth;
}

EXPORT_C void CContainerXml::CharacterDataL(const XML_Char * /*s*/, int /*len*/)
{
	// ignore
}

EXPORT_C void CContainerXml::EndElementL(const XML_Char *s)
{
	if (iDepth==2) {
		EndInnerElementL(s);
	}
}

EXPORT_C void CContainerXml::EndElement(const XML_Char *s)
{
	if (!iIgnoreRest) {
		TRAPD(err, EndElementL(s));
		if (err!=KErrNone) {
			ErrorInHandler(err);
		}
	}
	--iDepth;
	if (iDepth==0) {
		iParent->EndElement(s);
		iParser->SetHandler(iParent);
	}
}


EXPORT_C void CContainerXml::SetCurrentHandler(MNestedXmlHandler* aHandler)
{
	delete iCurrentHandler;
	iCurrentHandler=aHandler;
	iParser->SetHandler(aHandler);
}

EXPORT_C MNestedXmlHandler* CContainerXml::GetCurrentHandler()
{
	return iCurrentHandler;
}

EXPORT_C CContainerXml::~CContainerXml()
{
	delete iCurrentHandler;
}

EXPORT_C CPartContainerXml::CPartContainerXml(MNestedXmlHandler* aParent, CXmlParser* aParser,
					      HBufC*& aBuf, TBBCompoundData& aValue, TBool aCheckType) :
	CContainerXml(aParent, aParser, aBuf, aValue,  aCheckType), iCompoundValue(aValue)
{
}

EXPORT_C void CPartContainerXml::StartInnerElementL(const XML_Char *name, const XML_Char **atts)
{
	//RDebug::Print(_L("CPartContainerXml::StartInnerElementL"));

	TPtrC namep((const TUint16*)name);

	TUint partno;
	MBBData *p=iCompoundValue.GetPart(namep, TTypeName::IdFromAttributes(atts), partno);
	iCurrentPart=partno;
	if (p) {
		SetCurrentHandler(p->FromXmlL(this, iParser, iBuf, iCheckType) );
		GetCurrentHandler()->StartElement(name, atts);
	} else {
		User::Leave(KUnexpectedElement);
	}
}

EXPORT_C void CPartContainerXml::EndInnerElementL(const XML_Char *)
{
	//RDebug::Print(_L("CPartContainerXml::EndInnerElementL"));

	if (iCurrentPart>=0) iCompoundValue.ReadPart(iCurrentPart);
}

EXPORT_C void CPartContainerXml::Error(XML_Error Code, const XML_LChar * /*String*/, long /*ByteIndex*/)
{
	SetError(Code);
}

EXPORT_C void CPartContainerXml::SetError(TInt aError)
{
	iCurrentPart=-1;
	if (!iParent) User::Leave(aError);
	else iParent->SetError(aError);
}

class CSingleParserImpl : public CSingleParser, public MNestedXmlHandler {
public:
	MBBData* iData; 
	MNestedXmlHandler* iCurrentHandler;
	CXmlParser* iParser;
	HBufC* iBuf;
	TBool  iCheckType;
	
	void SetParser(CXmlParser* aParser) { iParser=aParser; }
	CSingleParserImpl(MBBData* aData, TBool  aCheckType) : iData(aData), iCheckType(aCheckType) { }
	void ConstructL() {
		iParser=CXmlParser::NewL(*this);
	}
	~CSingleParserImpl() { delete iBuf; delete iCurrentHandler; delete iParser; }

	virtual void ParseL(const TDesC8& aXml) {
		iParser->Parse( (char*)(aXml.Ptr()), aXml.Size(), 0);
	}

	virtual void StartElement(const XML_Char *name,
				const XML_Char **atts) {

		if (!iCurrentHandler) iCurrentHandler=iData->FromXmlL(this, iParser, iBuf, iCheckType);
		else User::Leave(KErrGeneral);

		if (iCurrentHandler) iCurrentHandler->StartElement(name, atts);
		iParser->SetHandler(iCurrentHandler);
	}
		
	virtual void EndElement(const XML_Char * /*name*/) {
	}

	virtual void CharacterData(const XML_Char * /*s*/,
				    int /*len*/) {
		User::Leave(KErrGeneral);
	}

	virtual void Error(XML_Error /*Code*/, const XML_LChar * /*String*/, long /*ByteIndex*/) {
		User::Leave(KErrInvalidXml);
	}
	virtual void SetError(TInt aError) { 
		User::Leave(aError); 
	}
};

EXPORT_C CSingleParser* CSingleParser::NewL(MBBData* aParseInto, TBool  aCheckType) {
	CSingleParserImpl *ret=new (ELeave) CSingleParserImpl(aParseInto, aCheckType);
	CleanupStack::PushL(ret);
	ret->ConstructL();
	CleanupStack::Pop();
	return ret;
}
