/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/


#ifndef CB_PHONEBOOK_H_INCLUDED
#define CB_PHONEBOOK_H_INCLUDED 1

#include <e32base.h>
#include <badesca.h>
#include <cntdef.h>
#include <cpbkcontactengine.h> 
#include <mpbkcontactdbobserver.h> 
#include <cpbkcontactchangenotifier.h> 
#include "db.h"
#include "symbian_tree.h"
#include "cb_presence.h"
#include "jabberdata.h"

class contact : public CBase 
{
public:
	TContactItemId	id;
	HBufC*		first_name;
	HBufC*		last_name;
	TInt		current_idx;
	bool		has_nick;
	CBBPresence* presence;
	void Name(HBufC*& Into, bool last_name_first);
	contact() {
		id=0;
		first_name=0;
		last_name=0;
		presence=0;
		has_nick=false;
	}

	static contact* NewL(TContactItemId i_id, const TDesC& i_first_name, const TDesC& i_last_name);
	void set_presence(CBBPresence* data);
	~contact();
};

class phonebook_observer {
public:
	virtual void before_change() = 0;
	virtual void contents_changed() = 0;
	virtual void exiting() = 0;
};

class phonebook_i {
public:
	phonebook_i() { }
	virtual ~phonebook_i() { }
	virtual void ConstructL() = 0;
	virtual CPbkContactEngine* get_engine() = 0;
	
	virtual TInt Count() = 0;
	virtual void set_observer(phonebook_observer* i_obs) = 0;

	virtual bool filter(const TDesC& substr, bool force=false) = 0;
	virtual TInt GetContactId(TInt Index) = 0;
	virtual TInt GetIndex(TInt ContactId) = 0;
	virtual contact * GetContact(TInt index) = 0;
	virtual void ReRead() = 0;
	virtual TBuf<100> ExportVCardToFile(TInt ContactId) = 0;

};


class CPresenceArray : public CBase, public MDesCArray {
public:
	static CPresenceArray* NewL(phonebook_i* aPhoneBook);
	virtual ~CPresenceArray();
};

class CNameArray : public CBase, public MDesCArray {
public:
	static CNameArray* NewL(phonebook_i* aPhoneBook);
	virtual ~CNameArray();
};

class phonebook : public CBase, MPbkContactDbObserver, public phonebook_i, public MContextBase,
	public MPresenceListener {
public:
	phonebook(MApp_context& Context, CJabberData& JabberData, CPresenceHolder& PresenceHolder);
	virtual ~phonebook();
	void ConstructL();

	//from phonebook_i
	virtual TInt Count();
	virtual TInt GetContactId(TInt Index);
	virtual TInt GetIndex(TInt ContactId);

	
	CPbkContactEngine* get_engine();
	bool filter(const TDesC& substr, bool force=false);
	virtual void set_observer(phonebook_observer* i_obs);
	phonebook_observer * get_observer();

	
	virtual void PresenceChangedL(TInt ContactId, CBBPresence* Info);
	void Notify(const TDesC& aMessage);
	virtual contact * GetContact(TInt index);
	CArrayFixFlat<contact*>* GetContacts();

	virtual TBuf<100> ExportVCardToFile(TInt ContactId);

	virtual void ReRead();

private:
	// MPbkContactDbObserver::
	void HandleDatabaseEventL(TContactDbObserverEvent aEvent); 

	void read_data(CPbkContactIter * iter);
	void read_db();
	void reset_contacts();
	
	CArrayFixFlat<contact*>* contacts;
	CArrayFixFlat<TContactItemId>* current_ids;
	
	CPbkContactEngine *eng;
	bool owns_engine;
	TBuf<20> previous_filter;
	phonebook_observer* obs;
	CPbkContactItem* contactitem; // used for getting phone no

	CPbkContactChangeNotifier* chnot;
	CGenericIntMap *iContactToIndex;
	CJabberData&	iJabberData;
	CPresenceHolder& iPresenceHolder;
};


class CPresenceUpdater : public MTimeOut, public CBase
{
public : 
	static CPresenceUpdater * NewL(phonebook_i & book);
	void ConstructL();
	virtual ~CPresenceUpdater();
	CPresenceUpdater(phonebook_i & book);


	void Start();
	void Stop();
	void Refresh();

public : // MTimeOut
	void expired(CBase*);

private:
	phonebook_i & book;
	CTimeOut * iWait;
	TBool iWaiting;

	const static TInt KTimeOut;
};

#endif
