#ifndef CONTEXT_CSD_PRESENCE_H_INCLUDED
#define CONTEXT_CSD_PRESENCE_H_INCLUDED 1

#include <csd_cell.h>
#include <csd_base.h>
#include <csd_idle.h>
#include <csd_profile.h>
#include <csd_bluetooth.h>
#include <csd_gps.h>

const TTypeName KPresenceType = { { CONTEXT_UID_SENSORDATAFACTORY }, 31, 1, 0 };
const TTupleName KPresenceTuple = { { CONTEXT_UID_CONTEXTSENSORS }, 31 };

class CBBPresence : public TBBCompoundData, public CBase {
public:
	TBBCellId	iCellId;
	TBBBaseInfo	iBaseInfo;
	TBBUserActive	iUserActive;
	TBBProfile	iProfile;
	CBBBtDeviceList*	iDevices;
	TGpsLine	iGps;
	TBBNeighbourhoodInfo iNeighbourhoodInfo;
	TBBShortString	iUserGiven;
	
	TBBTime		iSentTimeStamp;
	TBool		iSent;

	IMPORT_C void SetDevices(const CBBBtDeviceList* aList); // clones the list

	IMPORT_C void AddRef();
	IMPORT_C void Release();
	IMPORT_C void ConstructL();

        IMPORT_C virtual const TTypeName& Type() const;
	IMPORT_C virtual TBool Equals(const MBBData* aRhs) const;

	IMPORT_C static const TTypeName& StaticType();
	IMPORT_C const MBBData* Part(TUint aPartNo) const;
	IMPORT_C CBBPresence();
	IMPORT_C virtual const TDesC& StringSep(TUint aBeforePart) const;

	IMPORT_C bool operator==(const CBBPresence& aRhs) const;
	IMPORT_C MBBData* CloneL(const TDesC& Name) const;
	IMPORT_C CBBPresence& operator=(const CBBPresence& aRhs);

	IMPORT_C static CBBPresence* NewL();
	IMPORT_C ~CBBPresence();

private:
	TUint	iRefCount;
};

IMPORT_C TBool IsOutOfDate(TTime stamp, TInt freshness_interval = 10 /*in minutes*/);

IMPORT_C TBuf<8> TimeSinceStamp(TTime stamp, TInt minuteInterval=10);

IMPORT_C void PresenceToListBoxL(const CBBPresence* data, HBufC*& name,
				 HBufC* last_name, HBufC* first_name, const TDesC& prev, const TDesC& not_avail,
				 bool last_name_first=true);

#endif
