#include "csd_presence.h"
#include <icons.h>
#include <contextcommon.mbg>
#include <avkon.mbg>
#include <e32math.h>
#include <symbian_auto_ptr.h>

EXPORT_C CBBPresence* CBBPresence::NewL()
{
	auto_ptr<CBBPresence> ret(new (ELeave) CBBPresence);
	ret->ConstructL();
	return ret.release();
}
EXPORT_C void CBBPresence::AddRef()
{
	++iRefCount;
}

EXPORT_C void CBBPresence::Release()
{
	--iRefCount;
	if (iRefCount==0) delete this;
}

EXPORT_C const TTypeName& CBBPresence::Type() const
{
	return KPresenceType;
}

EXPORT_C TBool CBBPresence::Equals(const MBBData* aRhs) const
{
	const CBBPresence* rhs=bb_cast<CBBPresence>(aRhs);
	return (rhs && *this==*rhs);
}

EXPORT_C const TTypeName& CBBPresence::StaticType()
{
	return KPresenceType;
}

EXPORT_C void CBBPresence::ConstructL()
{
	iDevices=CBBBtDeviceList::NewL();
}

EXPORT_C const MBBData* CBBPresence::Part(TUint aPartNo) const
{
	switch(aPartNo) {
	case 0:
		return &iCellId;
		break;
	case 1:
		return &iBaseInfo;
		break;
	case 2:
		return &iUserActive;
		break;
	case 3:
		return &iProfile;
		break;
	case 7:
		// last so that if it is null, the others get done
		return iDevices;
		break;
	case 5:
		return &iGps;
		break;
	case 6:
		return &iNeighbourhoodInfo;
		break;
	case 4:
		return &iUserGiven;
		break;
	default:
		return 0;
	}
}

_LIT(KPresence, "presencev2");
_LIT(KUserGiven, "usergiven");
_LIT(KSent, "sent");

EXPORT_C CBBPresence::CBBPresence() : TBBCompoundData(KPresence), iUserGiven(KUserGiven), iCellId(KCell),
	iSentTimeStamp(KSent) { }

_LIT(KSpace, " ");

EXPORT_C const TDesC& CBBPresence::StringSep(TUint aBeforePart) const
{
	return KSpace;
}

EXPORT_C bool CBBPresence::operator==(const CBBPresence& aRhs) const
{
	return (iCellId==aRhs.iCellId &&
		iBaseInfo==aRhs.iBaseInfo &&
		iUserActive==aRhs.iUserActive &&
		iProfile==aRhs.iProfile &&
		iDevices->Equals(aRhs.iDevices) &&
		iGps==aRhs.iGps &&
		iNeighbourhoodInfo==aRhs.iNeighbourhoodInfo &&
		iUserGiven()==aRhs.iUserGiven() &&
		iSentTimeStamp()==aRhs.iSentTimeStamp() &&
		iSent==aRhs.iSent
		);
}

EXPORT_C MBBData* CBBPresence::CloneL(const TDesC& Name) const
{
	auto_ptr<CBBPresence> ret(new (ELeave) CBBPresence);
	*ret=*this;
	return ret.release();
}

EXPORT_C void CBBPresence::SetDevices(const CBBBtDeviceList* aList)
{
	MBBData* d=aList->CloneL(aList->Name());
	CBBBtDeviceList* l=bb_cast<CBBBtDeviceList>(d);
	if (!l) {
		delete d;
		User::Leave(KErrNotSupported);
	}
	delete iDevices;
	iDevices=l;
}

EXPORT_C CBBPresence& CBBPresence::operator=(const CBBPresence& aRhs)
{
	MBBData* d=aRhs.iDevices->CloneL(aRhs.iDevices->Name());
	CBBBtDeviceList* l=bb_cast<CBBBtDeviceList>(d);
	if (!l) {
		delete d;
		User::Leave(KErrNotSupported);
	}
	delete iDevices;
	iDevices=l;

	iCellId=aRhs.iCellId;
	iBaseInfo=aRhs.iBaseInfo;
	iUserActive=aRhs.iUserActive;
	iProfile=aRhs.iProfile;

	iGps=aRhs.iGps;
	iNeighbourhoodInfo=aRhs.iNeighbourhoodInfo;
	iUserGiven()=aRhs.iUserGiven();
	iSentTimeStamp()=aRhs.iSentTimeStamp();
	iSent=aRhs.iSent;

	return *this;
}

EXPORT_C TBool IsOutOfDate(TTime stamp, TInt freshness_interval)
{
	//const TInt KFreshnessInterval = 10; // in minutes 
	
	TTime now = TTime();
	now.HomeTime();

	TTimeIntervalMinutes m;
	now.MinutesFrom(stamp, m);
	if (m.Int() > freshness_interval)
	{
		return ETrue;	
	}
	else
	{
		return EFalse;
	}
}

EXPORT_C TBuf<8> TimeSinceStamp(TTime stamp, TInt minuteInterval)
{
	TTime now; now.HomeTime();
	TBuf<128> ret;
	
	TTimeIntervalDays days = now.DaysFrom(stamp);
	if (days.Int() >= 2)
	{
		ret.Append(_L(">2D"));
	}
	else if (days.Int() >= 1)
	{
		ret.Append(_L(">1D"));
	}
	else
	{
		TTimeIntervalHours hours;
		now.HoursFrom(stamp, hours);
		TTimeIntervalMinutes minutes;
		now.MinutesFrom(stamp, minutes);
				
		ret.Format(_L("%d:"), hours.Int());
				
		TInt min = (minutes.Int() - (hours.Int() * 60));
		if (minuteInterval==0) minuteInterval=10;

		TInt32 result = 0;
		TInt err = Math::Int(result, min/minuteInterval);
		if (err==KErrNone)
		{
			result = result*minuteInterval;	
		}
		else
		{
			result = min;
		}
		ret.AppendFormat(_L("%02d"), result);
	}
	return ret;
}


EXPORT_C void PresenceToListBoxL(const CBBPresence* data, HBufC*& name,
				 HBufC* last_name, HBufC* first_name, const TDesC& prev,
				 const TDesC& not_avail, bool last_name_first)
{
	//const MPresenceData* presence=data;

	
	/* length */
	TInt base_len=0;
	if ( data && data->iBaseInfo.iPreviousStay.IsSet() ) 
		base_len=data->iBaseInfo.iPreviousStay.iBaseName().Length();
	else if ( data && data->iBaseInfo.iCurrent.IsSet() ) 
		base_len=data->iBaseInfo.iCurrent.iBaseName().Length();

	TInt len=name->Length()+base_len+120;
	name->Des().Zero();
	while (len>name->Des().MaxLength()) {
		name=name->ReAllocL(name->Des().MaxLength()*2);
	}

	// - Name --------------------------------------------------------------

	if (last_name_first)
	{
		if (last_name) name->Des().Append(*last_name);
		if (last_name && last_name->Length()>0 && first_name && first_name->Length()>0) {
			name->Des().Append(_L(" "));
		}
		if (first_name) name->Des().Append(*first_name);
	}
	else
	{
		if (first_name) name->Des().Append(*first_name);
		if (first_name && first_name->Length()>0 && last_name && last_name->Length()>0) {
			name->Des().Append(_L(" "));
		}
		if (last_name) name->Des().Append(*last_name);
	}

	

	if (!data || ! data->iSent) {
	//if (!data ) {
		name->Des().Append(_L("\t"));
		name->Des().Append(not_avail);
		name->Des().AppendFormat(_L("\t0\t0\t0\t0\t0\t%02d"), GetIconIndex(EMbmAvkonQgn_indi_marked_add) );
		return;
	}


	if ( data->iUserGiven.iValue.Length() != 0 )
	{
		if ( name->Des().Length() > 6) {
			TBuf<6> temp;
			temp.Append(name->Des().Left(6));
			name->Des().Copy(temp);
			name->Des().Append(_L("."));
		}
		name->Des().Append(_L(" '"));
		name->Des().Append(data->iUserGiven.Value());
		name->Des().Append(_L("'"));
	}

	name->Des().Append(_L("\t"));	
	
	TBool out_of_date = IsOutOfDate(data->iSentTimeStamp());
		
	// - Location Info -----------------------------------------------------
	
	TTime stamp; stamp.HomeTime();
	
	TBool prev_known=data->iBaseInfo.iPreviousStay.IsSet();
	TBool current_known=data->iBaseInfo.iCurrent.IsSet();
	if (  prev_known && !current_known ) {
		name->Des().Append(_L("? ("));
		name->Des().Append(prev);
		name->Des().Append(_L(" "));
		name->Des().Append(data->iBaseInfo.iPreviousStay.iBaseName());
		stamp = data->iBaseInfo.iPreviousStay.iLeft();
	} else if (current_known) {
		name->Des().Append(data->iBaseInfo.iCurrent.iBaseName());
		stamp = data->iBaseInfo.iCurrent.iEntered();
	} else {
		name->Des().Append(_L("?"));
	}
	
	name->Des().Append(_L(" ("));
	name->Des().Append(TimeSinceStamp(stamp));
	name->Des().Append(_L(")"));
	
	if (  prev_known && !current_known) {
		name->Des().Append(_L(" )"));
	}
	
	// - User activity --------------------------------------------------
	if (data->iUserActive.iActive() )
	{
		if (out_of_date) 
		{
			TBuf<5> icon_int;
			icon_int.Format(_L("\t%02d"), GetIconIndex(EMbmContextcommonUser_inactive) );
			name->Des().Append(icon_int);
		}
		else
		{
			TBuf<5> icon_int;
			icon_int.Format(_L("\t%02d"), GetIconIndex(EMbmContextcommonUser_active) );
			name->Des().Append(icon_int);
		}
	} else 	{
		TTimeIntervalMinutes minutes;
		TTime now; now.HomeTime();
		
		now.MinutesFrom(data->iUserActive.iSince(), minutes);
		
		if (minutes.Int() <= 10)
		{
			TBuf<5> icon_int;
			icon_int.Format(_L("\t%02d"), GetIconIndex(EMbmContextcommonUser_inactive_lvl_1) );
			name->Des().Append(icon_int);
		}
		else if (minutes.Int() <= 60)
		{
			TBuf<5> icon_int;
			icon_int.Format(_L("\t%02d"), GetIconIndex(EMbmContextcommonUser_inactive_lvl_2) );
			name->Des().Append(icon_int);
		}
		else if (minutes.Int() <= 240)
		{
			TBuf<5> icon_int;
			icon_int.Format(_L("\t%02d"), GetIconIndex(EMbmContextcommonUser_inactive_lvl_3) );
			name->Des().Append(icon_int);
		}
		else
		{
			// no icon
			name->Des().Append(_L("\t0"));
		}
	}
	
	
	TInt ringtype=data->iProfile.iRingingType();
	// - Speaker Info -----------------------------------------------------
	if ( ringtype == TBBProfile::ERingingTypeSilent )
	{
		if (out_of_date) {
			TBuf<5> icon_int;
			icon_int.Format(_L("\t%02d"), GetIconIndex(EMbmContextcommonSpeaker_off_grey) );
			name->Des().Append(icon_int);	
		} else {
			TBuf<5> icon_int;
			icon_int.Format(_L("\t%02d"), GetIconIndex(EMbmContextcommonSpeaker_off) );
			name->Des().Append(icon_int);	
		}
		
	} else {
		if (out_of_date) {
			TBuf<5> icon_int;
			icon_int.Format(_L("\t%02d"), GetIconIndex(EMbmContextcommonSpeaker_on_grey) );
			name->Des().Append(icon_int);		
		} else {
			TBuf<5> icon_int;
			icon_int.Format(_L("\t%02d"), GetIconIndex(EMbmContextcommonSpeaker_on) );
			name->Des().Append(icon_int);				
		}
	}
	
	// - Vibrator Info -----------------------------------------------------
	if ( data->iProfile.iVibra() )
	{
		if (out_of_date) {
			TBuf<5> icon_int;
			icon_int.Format(_L("\t%02d"), GetIconIndex(EMbmContextcommonVibrator_on_grey) );
			name->Des().Append(icon_int);		
		} else {
			TBuf<5> icon_int;
			icon_int.Format(_L("\t%02d"), GetIconIndex(EMbmContextcommonVibrator_on) );
			name->Des().Append(icon_int);
			
		}
	} else {
		if (out_of_date) {
			TBuf<5> icon_int;
			icon_int.Format(_L("\t%02d"), GetIconIndex(EMbmContextcommonVibrator_off_grey) );
			name->Des().Append(icon_int);		
		} else {
			TBuf<5> icon_int;
			icon_int.Format(_L("\t%02d"), GetIconIndex(EMbmContextcommonVibrator_off) );
			name->Des().Append(icon_int);	
		}
	}

	// - NeighbourhoodInfo -------------------
	// - Buddies -----------------------------
	TBuf<5> icon_int;
	TInt idx = 0;

	switch (data->iNeighbourhoodInfo.iBuddies())
	{
		case 0:
			idx = 0;
			break;
		case 1:
			if (out_of_date) {
				idx = GetIconIndex(EMbmContextcommonBuddy1_grey);
			} else {
				idx = GetIconIndex(EMbmContextcommonBuddy1);
			}
			break;
		case 2:
			if (out_of_date) {
				idx = GetIconIndex(EMbmContextcommonBuddy2_grey);
			} else {
				idx = GetIconIndex(EMbmContextcommonBuddy2);
			}
			break;
		case 3:
			if (out_of_date) {
				idx = GetIconIndex(EMbmContextcommonBuddy3_grey);
			} else {
				idx = GetIconIndex(EMbmContextcommonBuddy3);
			}
			break;
		default:
			if (out_of_date) {
				idx = GetIconIndex(EMbmContextcommonBuddy4_grey);
			} else {
				idx = GetIconIndex(EMbmContextcommonBuddy4);
			}
			break;
	}
	icon_int.Format(_L("\t%02d"), idx );
	name->Des().Append(icon_int);
	
	// - Other_phones -----------------------------
	idx=0;
	icon_int.Zero();
	
	switch (data->iNeighbourhoodInfo.iOtherPhones())
	{
		case 0:
			idx = 0;
			break;
		case 1:
			if (out_of_date) {
				idx = GetIconIndex(EMbmContextcommonOther1_grey);
			} else {
				idx = GetIconIndex(EMbmContextcommonOther1);
			}
			break;
		case 2:
			if (out_of_date) {
				idx = GetIconIndex(EMbmContextcommonOther2_grey);
			} else {
				idx = GetIconIndex(EMbmContextcommonOther2);
			}
			break;
		case 3:
			if (out_of_date) {
				idx = GetIconIndex(EMbmContextcommonOther3_grey);
			} else {
				idx = GetIconIndex(EMbmContextcommonOther3);
			}
			break;
		default:
			if (out_of_date) {
				idx = GetIconIndex(EMbmContextcommonOther4_grey);
			} else {
				idx = GetIconIndex(EMbmContextcommonOther4);
			}
			break;
	}
	icon_int.Format(_L("\t%02d"), idx );
	name->Des().Append(icon_int);

	// - marked icon - has to be the final one! -------------------------------------

	idx = GetIconIndex(EMbmAvkonQgn_indi_marked_add);
	icon_int.Format(_L("\t%02d"), idx );
	name->Des().Append(icon_int);
}

EXPORT_C CBBPresence::~CBBPresence()
{
	delete iDevices;
}