#include "log_shareddata.h"

#include <symbian_auto_ptr.h>
#include <checkedactive.h>

#define NO_SHAREDDATACLIENT_H

#ifndef NO_SHAREDDATACLIENT_H
#include <shareddataclient.h>
#else
class MSharedDataNotifyHandler { };
#endif

#define MAX_ERRORS	10

#include <csd_system.h>

class CLog_SharedDataImpl : public CLog_SharedData, public MSharedDataNotifyHandler, public CBase {
public:
	virtual ~CLog_SharedDataImpl();
private:
	CLog_SharedDataImpl(MApp_context& Context, const TDesC& name, const TTupleName& aTupleName);
	void ConstructL(TUid Uid, const TDesC& aKeyName);

	virtual void HandleNotifyL( const TUid aUid, const TDesC& aKey,
                                const TDesC& aValue );

#ifndef NO_SHAREDDATACLIENT_H
	RSharedDataClient iAgent;
	TInt		iErrorCount;

	TUid		iUid;
	TBuf<50>	iKeyName;
#endif

	TBBLongString	iValue;

	friend class CLog_SharedData;
};

CLog_SharedData* CLog_SharedData::NewL(MApp_context& Context, const TDesC& name, TUid Uid, 
				       const TDesC& aKeyName, const TTupleName& aTupleName)
{
	auto_ptr<CLog_SharedDataImpl> ret(new (ELeave) CLog_SharedDataImpl(Context, name, aTupleName));
	ret->ConstructL(Uid, aKeyName);
	return ret.release();
}

CLog_SharedData::CLog_SharedData(MApp_context& Context, const TDesC& name, const TTupleName& aTupleName) : 
	Mlog_base_impl(Context, name, aTupleName)
{
}

CLog_SharedDataImpl::~CLog_SharedDataImpl()
{
#ifndef NO_SHAREDDATACLIENT_H
	iAgent.CancelNotify(iUid, &iKeyName);
	iAgent.Close();
#endif
}

CLog_SharedDataImpl::CLog_SharedDataImpl(MApp_context& Context, const TDesC& name, const TTupleName& aTupleName) : 
	CLog_SharedData(Context, name, aTupleName), 
#ifndef NO_SHAREDDATACLIENT_H
	iAgent(this), 
#endif
	iValue(name)
{
}

void CLog_SharedDataImpl::ConstructL(TUid Uid, const TDesC& aKeyName)
{
	Mlog_base_impl::ConstructL();
#ifndef NO_SHAREDDATACLIENT_H
	iUid=Uid;
	iKeyName=aKeyName;
	User::LeaveIfError(iAgent.Connect());
	User::LeaveIfError(iAgent.Assign(iUid));

	iAgent.GetString(iKeyName, iValue());
#else
	iValue()=_L("not supported");
#endif
	iEvent.iStamp()=GetTime();
	iEvent.iData()=&iValue; iEvent.iData.SetOwnsValue(EFalse);

#ifndef NO_SHAREDDATACLIENT_H
	User::LeaveIfError(iAgent.NotifyChange(iUid));
#endif
}

void CLog_SharedDataImpl::HandleNotifyL( const TUid aUid, const TDesC& aKey,
                        const TDesC& aValue )
{
	iValue()=aKey;
	iValue().Append(_L(" "));
	iValue().Append(aValue);
	post_new_value(&iValue);
}